@@ -13,7 +13,7 @@
 #' @export
 #'
 #' @examples
-#'library(nnet)
+#' library(nnet)
 #' library(MASS)
 #'
 #' dataset <- data.frame(y = c(rep("a", 10), rep("b", 10), rep("c", 10)),
@@ -25,29 +25,14 @@
 #'
 #' fdi1 <- mnl_fd2_ova(model = mod, data = dataset,
 #'                     xvari = "x1",
-#'                     value1 = min(dataset$x1), value2 = max(dataset$x1))
+#'                     value1 = min(dataset$x1),
+#'                     value2 = max(dataset$x1),
+#'                     nsim = 10)
 #'
 #'
-#' \donttest{
-#' library(foreign)
-#' library(nnet)
-#' library(MASS)
-#'
-#' ml <- read.dta("https://stats.idre.ucla.edu/stat/data/hsbdemo.dta")
-#'
-#' ml$prog2 <- relevel(ml$prog, ref = "academic")
-#' ml$female2 <- as.numeric(ml$female == "female")
-#'
-#' mod1 <- multinom(prog2 ~ female2 + read + write + math + science,
-#'                  Hess = TRUE, data = ml)
-#'
-#' fd1 <- mnl_fd2_ova(model = mod1, data = ml,
-#'                    xvari = "math",
-#'                    value1 = min(ml$math), value2= max(ml$math),
-#'                    nsim = 1000)
-#' }
 #'
 #' @importFrom stats coef na.omit quantile
+#' @importFrom utils setTxtProgressBar txtProgressBar
 #' @importFrom MASS mvrnorm
 
 
@@ -81,6 +66,16 @@ mnl_fd2_ova <- function(model,
     stop("Please supply values to compute differences")
   }
 
+  if (is.null(model$Hessian) == TRUE) {
+    stop("There is no Hessian matrix. Please specify Hess = TRUE in your multinom() call.")
+  }
+
+  # Names of variables in model (without the "list" character in the vector)
+  variables <- as.character(attr(model$terms, "variables"))[-1]
+
+  if(!(xvari %in% variables) == TRUE){
+    stop("x-variable is not an independent variable in the model. There might be a typo.")
+  }
 
   # Create list that is returned in the end.
   output <- list()
@@ -113,10 +108,6 @@ mnl_fd2_ova <- function(model,
 
   nseq <- length(variation)
 
-
-  # Names of variables in model (without the "list" character in the vector)
-  variables <- as.character(attr(model$terms, "variables"))[-1]
-
   # Name of independent variables
   iv <- variables[2:length(variables)]
   output[["IV"]] <- iv
@@ -196,7 +187,12 @@ mnl_fd2_ova <- function(model,
   # Prepare array of observed values:
   ovaV <- array(NA, c(obs, nsim, nseq, J))
 
+  # Add progress bar
+  pb_multiplication <- txtProgressBar(min = 0, max = nseq, initial = 0)
+
   # Loop over all scenarios
+  cat("Multiplying values with simulated estimates:\n")
+
   for(i in 1:nseq){
     ovaV[, , i, 1] <- apply(matrix(0,
                                    nrow = nsim,
@@ -212,10 +208,16 @@ mnl_fd2_ova <- function(model,
                                      "], 1, function(s) ovacases[,, i] %*% s)"))
       eval(element)
     }
+
+    # Progress bar:
+    setTxtProgressBar(pb_multiplication, i)
   }
 
   # Multinomial link function:
 
+  pb_link <- txtProgressBar(min = 0, max = nseq, initial = 0)
+  cat("\nApplying link function:\n")
+
   # 1. Part: Sum over cases
   Sexp <- apply(ovaV, c(1, 2, 3), function(x) sum(exp(x)))
 
@@ -227,6 +229,8 @@ mnl_fd2_ova <- function(model,
     for (m in 1:J) {
       P[, m, l] <- apply(exp(ovaV[, , l, m])/Sexp[, , l], 2, mean)
     }
+
+    setTxtProgressBar(pb_link, l)
   }
 
   output[["P"]] <- P
@@ -247,7 +251,9 @@ mnl_fd2_ova <- function(model,
 
 
   # Aggregate
+
   start <- 1
+
   for (i in 1:J) {
     end <- i*length(variation)
     plotdat[c(start:end), "mean"] <- apply(P[, i,], 2, mean)
@@ -289,5 +295,6 @@ mnl_fd2_ova <- function(model,
 
   output[["plotdata_fd"]] <- plotdat_fd
 
+  cat("\nDone!\n\n")
   return(output)
 }
